Imports System.Threading
Imports Roslyn.Compilers
Imports Roslyn.Compilers.Common
Imports Roslyn.Compilers.VisualBasic
Imports Roslyn.Services
Imports Roslyn.Services.Editor

Namespace FindingDateTimeNow
    <ExportCodeIssueProvider("FindingDateTimeNow", LanguageNames.VisualBasic)>
    Public NotInheritable Class FindingDateTimeNowCodeIssueProvider
        Implements ICodeIssueProvider

        Public Function GetIssues(ByVal document As IDocument, ByVal node As CommonSyntaxNode, Optional ByVal cancellationToken As CancellationToken = Nothing) As IEnumerable(Of CodeIssue) Implements ICodeIssueProvider.GetIssues
            Dim memberNode = TryCast(node, MemberAccessExpressionSyntax)

            If memberNode.OperatorToken.Kind = SyntaxKind.DotToken AndAlso memberNode.Name.Identifier.ValueText = "Now" Then
                Dim symbol = document.GetSemanticModel.GetSymbolInfo(memberNode.Name).Symbol

                If symbol IsNot Nothing AndAlso symbol.ContainingType.ToDisplayString = Values.ExpectedContainingDateTimeTypeDisplayString AndAlso symbol.ContainingAssembly.ToDisplayString.Contains(Values.ExpectedContainingAssemblyDisplayString) Then
                    Return {New CodeIssue(CodeIssueKind.Error, memberNode.Name.Span, "Do not use DateTime.Now", New ChangeNowToUtcNowCodeAction(document, memberNode))}
                End If
            End If

            Return Nothing
        End Function

        Public Function GetIssues(ByVal document As IDocument, ByVal token As CommonSyntaxToken, Optional ByVal cancellationToken As CancellationToken = Nothing) As IEnumerable(Of CodeIssue) Implements ICodeIssueProvider.GetIssues
            Throw New NotImplementedException
        End Function

        Public ReadOnly Property SyntaxNodeTypes As IEnumerable(Of Type) Implements Roslyn.Services.ISyntaxMetadata.SyntaxNodeTypes
            Get
                Return {GetType(MemberAccessExpressionSyntax)}
            End Get
        End Property

        Public ReadOnly Property SyntaxTokenKinds As IEnumerable(Of Integer) Implements Roslyn.Services.ISyntaxMetadata.SyntaxTokenKinds
            Get
                Return Nothing
            End Get
        End Property
    End Class
End Namespace
