Imports Roslyn.Compilers
Imports Roslyn.Compilers.Common
Imports Roslyn.Compilers.CSharp
Imports Roslyn.Services
Imports Roslyn.Services.Editor
Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks

Namespace FindingDateTimeNow
    <ExportCodeIssueProvider("FindingNewDateTime", LanguageNames.VisualBasic)>
    Public NotInheritable Class FindingNewDateTimeCodeIssueProvider
        Implements ICodeIssueProvider

        Public Function GetIssues(ByVal document As IDocument, ByVal token As CommonSyntaxToken, Optional ByVal cancellationToken As CancellationToken = Nothing) As IEnumerable(Of CodeIssue) Implements ICodeIssueProvider.GetIssues
            Throw New NotImplementedException
        End Function

        Public Function GetIssues(ByVal document As IDocument, ByVal node As CommonSyntaxNode, Optional ByVal cancellationToken As CancellationToken = Nothing) As IEnumerable(Of CodeIssue) Implements ICodeIssueProvider.GetIssues
            Dim creationNode = TryCast(node, ObjectCreationExpressionSyntax)
            Dim creationNameNode = TryCast(creationNode.Type, IdentifierNameSyntax)

            If creationNameNode IsNot Nothing AndAlso creationNameNode.Identifier.ValueText = "DateTime" Then
                Dim model = document.GetSemanticModel
                Dim creationSymbol = model.GetSymbolInfo(creationNode).Symbol

                If creationSymbol IsNot Nothing AndAlso creationSymbol.ContainingType.ToDisplayString = Values.ExpectedContainingDateTimeTypeDisplayString AndAlso creationSymbol.ContainingAssembly.ToDisplayString.Contains(Values.ExpectedContainingAssemblyDisplayString) Then
                    Dim argument = FindingNewDateTimeCodeIssueProvider.GetInvalidArgument(creationNode, model)

                    If argument IsNot Nothing Then
                        If argument.Item2.Name = "Local" OrElse argument.Item2.Name = "Unspecified" Then
                            Return {New CodeIssue(CodeIssueKind.Error, argument.Item1.Span, "Do not use DateTimeKind.Local or DateTimeKind.Unspecified", New ChangeDateTimeKindToUtcCodeAction(document, argument.Item1))}
                        End If
                    Else
                        Return {New CodeIssue(CodeIssueKind.Error, creationNode.Span, "You must use a DateTime constuctor that takes a DateTimeKind")}
                    End If
                End If
            End If

            Return Nothing
        End Function

        Private Shared Function GetInvalidArgument(ByVal creationToken As ObjectCreationExpressionSyntax, ByVal model As ISemanticModel) As Tuple(Of ArgumentSyntax, ISymbol)
            For Each argument In creationToken.ArgumentList.Arguments
                If TypeOf argument.Expression Is MemberAccessExpressionSyntax Then
                    Dim argumentSymbolNode = model.GetSymbolInfo(argument.Expression).Symbol

                    If argumentSymbolNode.ContainingType.ToDisplayString = Values.ExpectedContainingDateTimeKindTypeDisplayString Then
                        Return New Tuple(Of ArgumentSyntax, ISymbol)(argument, argumentSymbolNode)
                    End If
                End If
            Next argument

            Return Nothing
        End Function

        Public ReadOnly Property SyntaxNodeTypes As IEnumerable(Of Type) Implements Roslyn.Services.ISyntaxMetadata.SyntaxNodeTypes
            Get
                Return {GetType(ObjectCreationExpressionSyntax)}
            End Get
        End Property

        Public ReadOnly Property SyntaxTokenKinds As IEnumerable(Of Integer) Implements Roslyn.Services.ISyntaxMetadata.SyntaxTokenKinds
            Get
                Return Nothing
            End Get
        End Property
    End Class
End Namespace
